package com.muyu.customer.business.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.muyu.customer.business.domain.req.FaultCodeEditReq;
import com.muyu.customer.business.domain.req.FaultCodeQueryReq;
import com.muyu.customer.business.domain.req.FaultCodeSaveReq;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.annotation.Excel;
import com.muyu.common.core.web.domain.BaseEntity;

/**
 * 车辆故障码对象 fault_code
 *
 * @author chx
 * @date 2024-06-20
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("fault_code")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "FaultCode", description = "车辆故障码")
public class FaultCode extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** 主键 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "主键", value = "主键")
    private Long id;

    /** 故障码 */
    @Excel(name = "故障码")
    @ApiModelProperty(name = "故障码", value = "故障码")
    private String faultCode;

    /** 故障标签 */
    @Excel(name = "故障标签")
    @ApiModelProperty(name = "故障标签", value = "故障标签")
    private String faultLabel;

    /** 故障位 */
    @Excel(name = "故障位")
    @ApiModelProperty(name = "故障位", value = "故障位")
    private Long faultBit;

    /** 故障值 */
    @Excel(name = "故障值")
    @ApiModelProperty(name = "故障值", value = "故障值")
    private Long faultValue;

    /**
    * 查询构造器
    */
    public static FaultCode queryBuild( FaultCodeQueryReq faultCodeQueryReq){
        return FaultCode.builder()
                .faultCode(faultCodeQueryReq.getFaultCode())
                .build();
    }

    /**
    * 添加构造器
    */
    public static FaultCode saveBuild(FaultCodeSaveReq faultCodeSaveReq){
        return FaultCode.builder()
                .faultCode(faultCodeSaveReq.getFaultCode())
                .faultLabel(faultCodeSaveReq.getFaultLabel())
                .faultBit(faultCodeSaveReq.getFaultBit())
                .faultValue(faultCodeSaveReq.getFaultValue())
                .build();
    }

    /**
    * 修改构造器
    */
    public static FaultCode editBuild(Long id, FaultCodeEditReq faultCodeEditReq){
        return FaultCode.builder()
            .id(id)
                .faultCode(faultCodeEditReq.getFaultCode())
                .faultLabel(faultCodeEditReq.getFaultLabel())
                .faultBit(faultCodeEditReq.getFaultBit())
                .faultValue(faultCodeEditReq.getFaultValue())
                .build();
    }

}
