package com.muyu.customer.business.consumer;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.customer.business.domain.FaultRecord;
import com.muyu.customer.business.saas.holder.DynamicDataSourceHolder;
import com.muyu.customer.business.service.FaultRecordService;
import org.springframework.amqp.core.ExchangeTypes;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * rabbitMQ消费者 Task
 *
 * @author DeKangLiu
 * Date 2024/6/20 22:33
 */
@Component
@Log4j2
public class RabbitConsumer {
    @Autowired
    private FaultRecordService faultRecordService;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @RabbitListener(bindings = {@QueueBinding(value = @Queue(name = "error"),
            exchange = @Exchange(name = "carnet_fault_event", type = ExchangeTypes.DIRECT))})
public void consumptionStartMessage(Message message, Channel channel) {
    try {
//        DynamicDataSourceHolder.setDynamicDataSourceKey("liu_"+42);
        log.info("接收到消息：{}", new String(message.getBody()));
        String body= new String(message.getBody());
        String[] split = body.split("_");
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = dateFormat.parse(split[3]);
        String s = redisTemplate.opsForValue().get(split[0]+"1");
        System.out.println(s);
        //选择数据源，切换数据源，
        DynamicDataSourceHolder.setDynamicDataSourceKey("liu_"+s);
        FaultRecord build = FaultRecord.builder()
                .vin(split[0])
                .faultCode(split[1])
                .faultLevel(split[2])
                .startTime(date)
                .build();
        log.info("123：{}",build);
        faultRecordService.save(build);
        // 移除数据源，
        DynamicDataSourceHolder.removeDynamicDataSourceKey();
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        log.info("消费成功！数据源为：{}",message);

    } catch (Exception e) {
        log.info("消费失败，{}",e.getMessage());
        try {
            //回退消息
            channel.basicReject(message.getMessageProperties().getDeliveryTag(),false);
            log.info("回退成功");
        }catch (IOException ex){
            log.info("回退失败:{}",ex.getMessage());
        }
        throw new RuntimeException(e);
    }
}
    @RabbitListener(bindings = {@QueueBinding(value = @Queue(name = "success"),
            exchange = @Exchange(name = "carnet_fault_event", type = ExchangeTypes.DIRECT))})
    public void successMessage(Message message, Channel channel) {
        try {
            log.info("恢复正常消息：{}", new String(message.getBody()));
            String body= new String(message.getBody());
            String[] split = body.split("_");
            DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date = dateFormat.parse(split[3]);
            String s = redisTemplate.opsForValue().get(split[0]+"1");
            System.out.println(s);
            //选择数据源，切换数据源，
            DynamicDataSourceHolder.setDynamicDataSourceKey("liu_"+s);
            FaultRecord build = FaultRecord.builder()
                    .vin(split[0])
                    .faultCode(split[1])
                    .faultLevel(split[2])
                    .startTime(date)
                    .build();
            FaultRecord faultRecordOne = faultRecordService.selectFault(build);
            log.info("查询到的故障为：{}",faultRecordOne);
            faultRecordOne.setEndTime(faultRecord.getEndTime());
            // 进行修改故障表
            faultRecordService.updateByFaultRecord(faultRecordOne);
            // 移除数据源，
            DynamicDataSourceHolder.removeDynamicDataSourceKey();
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(), true);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

}
