package com.muyu.customer.business.saas.datasource;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson.JSON;
import com.muyu.common.core.utils.SpringUtils;

import com.muyu.common.redis.service.RedisService;
import com.muyu.customer.business.saas.factory.DruidDataSourceFactory;
import com.muyu.customer.business.saas.domain.model.DataSourceInfo;
import com.muyu.customer.business.saas.role.DynamicDataSource;
import com.muyu.net.working.remote.RemoteDatasourceService;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import com.muyu.net.working.domain.DataSource;
import java.io.IOException;
import java.util.*;

/**
 * 多数据源 ManyDataSource
 *
 * @author DeKangLiu
 * Date 2024/6/3 20:01
 */
@Component
@Log4j2
@AutoConfigureBefore(RedisService.class)
public class ManyDataSource {

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private RemoteDatasourceService remoteDatasourceService;

    @Autowired
    private RabbitTemplate rabbitTemplate;
    @RabbitListener(queuesToDeclare = {@Queue(name = "liu-vehicle-exchange")})
    public void smsConfig(String msg, Message message, Channel channel){
        //获取消息的ID
        String messageId = message.getMessageProperties().getMessageId();
        try {
            //添加消息id到redis set集合中  添加成功返回1 表示未消费    添加失败返回0 表示已消费
            Long count = redisTemplate.opsForSet().add("messageId", messageId);
            //添加成功  正常消费信息
            if (count == 1) {
                log.info("开始消费");
                druidData(msg);
                //确认消费
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                log.info("消费成功");
            }
        } catch (Exception e) {
            log.info("消费失败，尝试重连");
            try {
                //回退消息,尝试重连
                druidData(msg);
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),false);
                log.info("消费失败");
            } catch (IOException ex) {
                //回退失败
                log.info("消费异常");
            }
        }
    }
    private void druidData(String msg) {
        DataSource entInfo= JSON.parseObject(msg, DataSource.class);
        log.info("消息为：{}",entInfo);
        DruidDataSourceFactory druidDataSourceFactory = SpringUtils.getBean(DruidDataSourceFactory.class);
        DynamicDataSource dynamicDataSource = SpringUtils.getBean(DynamicDataSource.class);
        DataSourceInfo dataSourceInfo = DataSourceInfo.hostAndPortBuild(entInfo.getEnterpriseCode(), entInfo.getIp(), entInfo.getPort());
        DruidDataSource druidDataSource = druidDataSourceFactory.create(dataSourceInfo);
        dynamicDataSource.put(dataSourceInfo.getKey(), druidDataSource);
    }

    @Lazy
    private List<DataSource> dataSourceInfoList(){
        List<DataSource> databaseNameList = new ArrayList<>(){{
            add(DataSource.builder()
                    .enterpriseCode("liu_44")
                    .ip("192.168.116.129")
                    .port(3350)
                    .build());
        }};
        List<DataSource> list = remoteDatasourceService.listAll();
        System.out.println(list);
        databaseNameList.addAll(list);
        return databaseNameList;
    }

    @Bean
    @Primary
    public DynamicDataSource dynamicDataSource(DruidDataSourceFactory druidDataSourceFactory) {
        //查企业
        Map<Object, Object> dataSourceMap = new HashMap<>();
        dataSourceInfoList()
                .stream()
                .map(entInfo-> DataSourceInfo.hostAndPortBuild(entInfo.getEnterpriseCode(),entInfo.getIp(),entInfo.getPort()))
                .forEach(dataSourceInfo -> {
                    dataSourceMap.put(dataSourceInfo.getKey(), druidDataSourceFactory.create(dataSourceInfo));
                });
        //设置动态数据源
        DynamicDataSource dynamicDataSource = new DynamicDataSource();
//        dynamicDataSource.setDefaultTargetDataSource(masterDataSource());
        dynamicDataSource.setTargetDataSources(dataSourceMap);
        //将数据源信息备份在defineTargetDataSources中
        dynamicDataSource.setDefineTargetDataSources(dataSourceMap);
        return dynamicDataSource;
    }
}
