package com.muyu.customer.business.saas.role;


import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.SysUser;
import com.muyu.common.system.remote.RemoteUserService;
import com.muyu.customer.business.saas.holder.DynamicDataSourceHolder;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

/**
 * 多数据源切面
 *
 * @author DeKangLiu
 * Date 2024/6/3 20:12
 */
@Aspect
@Component
public class DataSourceAsp {
    @Lazy
    @Autowired
    private RemoteUserService remoteUserService;

    @Pointcut("execution(public * com.muyu.customer.business.controller.*Controller.*(..))")
    public void pointcut () {
    }


    /**
     * 的每一個方法執行之前 執行的處理
     */
    @Before("pointcut()")
    public void beforeMethod() {
        Integer enterpriseId = SecurityUtils.getLoginUser().getSysUser().getEnterpriseId();
        DynamicDataSourceHolder.setDynamicDataSourceKey("liu_"+enterpriseId);
    }
    /**
     * 的每一個方法執行之后 執行的處理
     * 无论正常还是异常终了
     * 不能接受到返回值
     */
    @After("pointcut()")
    public void afterMethod() {
        DynamicDataSourceHolder.removeDynamicDataSourceKey();
    }
}
