package com.muyu.net.working.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.net.working.domain.DataSource;
import com.muyu.net.working.domain.Enterprise;
import com.muyu.net.working.service.DataSourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 多数据源控制层 DataSourceController
 *
 * @author DeKangLiu
 * Date 2024/6/22 23:07
 */
@Api(tags = "多数据源")
@Log4j2
@RestController
@RequestMapping("/datasource")
public class DataSourceController extends BaseController {
    @Autowired
    private DataSourceService dataSourceService;

    @ApiOperation("获取多数据源列表")
    @GetMapping("/list")
    public Result<TableDataInfo<DataSource>> list(DataSource dataSource) {
        startPage();
        List<DataSource> list = dataSourceService.list(dataSource);
        return getDataTable(list);
    }

    @GetMapping("/listAll")
    public List<DataSource> listAll() {
        List<DataSource> list = dataSourceService.list();
        return list;
    }

    @PostMapping
    @ApiOperation("新增多数据源")
    public Result<String> add(@RequestBody DataSource dataSource) {
        return toAjax(dataSourceService.save(dataSource));
    }
}
