package com.muyu.net.working.service.impl;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.alibaba.fastjson.JSON;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.net.working.domain.DataSource;
import com.muyu.net.working.domain.req.EnterpriseSaveReq;
import com.muyu.net.working.service.DataSourceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.net.working.mapper.EnterpriseMapper;
import com.muyu.net.working.domain.Enterprise;
import com.muyu.net.working.service.EnterpriseService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 车辆运营平台Service业务层处理
 *
 * @author muyu
 * @date 2024-06-05
 */
@Slf4j
@Service
public class EnterpriseServiceImpl extends ServiceImpl<EnterpriseMapper, Enterprise>  implements EnterpriseService {

    @Autowired
    private EnterpriseMapper enterpriseMapper;
    @Autowired
    private DataSourceService dataSourceService;
    @Autowired
    private RabbitTemplate rabbitTemplate;
    /**
     * 查询车辆运营平台列表
     *
     * @param enterprise 车辆运营平台
     * @return 车辆运营平台
     */
    @Override
    public List<Enterprise> list(Enterprise enterprise) {
        LambdaQueryWrapper<Enterprise> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(enterprise.getEbterpriseName())){
            queryWrapper.like(Enterprise::getEbterpriseName, enterprise.getEbterpriseName());
        }

        if (ObjUtils.notNull(enterprise.getLegalPerson())){
            queryWrapper.eq(Enterprise::getLegalPerson, enterprise.getLegalPerson());
        }

        if (ObjUtils.notNull(enterprise.getBusinessLincenseNumber())){
            queryWrapper.eq(Enterprise::getBusinessLincenseNumber, enterprise.getBusinessLincenseNumber());
        }

        if (ObjUtils.notNull(enterprise.getEstabinessDate())){
            queryWrapper.eq(Enterprise::getEstabinessDate, enterprise.getEstabinessDate());
        }

        if (ObjUtils.notNull(enterprise.getBusinessScope())){
            queryWrapper.eq(Enterprise::getBusinessScope, enterprise.getBusinessScope());
        }

        if (ObjUtils.notNull(enterprise.getAddress())){
            queryWrapper.eq(Enterprise::getAddress, enterprise.getAddress());
        }

        if (ObjUtils.notNull(enterprise.getContactPhone())){
            queryWrapper.eq(Enterprise::getContactPhone, enterprise.getContactPhone());
        }

        if (ObjUtils.notNull(enterprise.getEmail())){
            queryWrapper.eq(Enterprise::getEmail, enterprise.getEmail());
        }

        if (ObjUtils.notNull(enterprise.getEnterpriseStatus())){
            queryWrapper.eq(Enterprise::getEnterpriseStatus, enterprise.getEnterpriseStatus());
        }

        if (ObjUtils.notNull(enterprise.getRegistrationDate())){
            queryWrapper.eq(Enterprise::getRegistrationDate, enterprise.getRegistrationDate());
        }

        if (ObjUtils.notNull(enterprise.getCertificationId())){
            queryWrapper.eq(Enterprise::getCertificationId, enterprise.getCertificationId());
        }

        if (ObjUtils.notNull(enterprise.getAuthenticationDate())){
            queryWrapper.eq(Enterprise::getAuthenticationDate, enterprise.getAuthenticationDate());
        }

        if (ObjUtils.notNull(enterprise.getServiceLevel())){
            queryWrapper.eq(Enterprise::getServiceLevel, enterprise.getServiceLevel());
        }

        if (ObjUtils.notNull(enterprise.getOpenServerId())){
            queryWrapper.eq(Enterprise::getOpenServerId, enterprise.getOpenServerId());
        }

        if (ObjUtils.notNull(enterprise.getAddServerId())){
            queryWrapper.eq(Enterprise::getAddServerId, enterprise.getAddServerId());
        }

        return list(queryWrapper);
    }

    @Override
    public Enterprise selectByName(String name) {
        return enterpriseMapper.selectByName(name);
    }

    @Override
    public void authentication(Enterprise enterprise) {
        enterpriseMapper.authentication(enterprise);
        try {
            postTest(enterprise);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void addEnterprise(EnterpriseSaveReq enterpriseSaveReq) {
        enterpriseMapper.addEnterprise(enterpriseSaveReq);
    }


    public void postTest(Enterprise enterprise) throws Exception {
        // 1.请求URL
        String postUrl = "http://192.168.116.129:10006/webhook/%E6%96%B0%E5%BB%BA%E4%BC%81%E4%B8%9A%E7%AE%A1%E7%90%86";
        // 2.请求参数JSON格式
//      enterprise.getId();
        Map<String, String> parammap = new HashMap<>();
        parammap.put("enterpriseId", "liu-"+enterprise.getId());
        parammap.put("mysqlPort", String.valueOf(3306+enterprise.getId()));
        String json = JSON.toJSONString(parammap);
        // 3.创建连接与设置连接参数
        URL urlObj = new URL(postUrl);
        HttpURLConnection httpConn = (HttpURLConnection) urlObj.openConnection();
        httpConn.setRequestMethod("POST");
        httpConn.setRequestProperty("Charset", "UTF-8");
        // POST请求且JSON数据,必须设置
        httpConn.setRequestProperty("Content-Type", "application/json");
        // 打开输出流,默认是false
        httpConn.setDoOutput(true);
        // 打开输入流,默认是true,可省略
        httpConn.setDoInput(true);
        // 4.从HttpURLConnection获取输出流和写数据
        OutputStream oStream = httpConn.getOutputStream();
        oStream.write(json.getBytes());
        oStream.flush();
        // 5.发起http调用(getInputStream触发http请求)
        if (httpConn.getResponseCode() != 200) {
            throw new Exception("调用服务端异常.");
        }
        // 6.从HttpURLConnection获取输入流和读数据
        BufferedReader br = new BufferedReader(
                new InputStreamReader(httpConn.getInputStream()));
        String resultData = br.readLine();
        System.out.println("从服务端返回结果: " + resultData);
        DataSource build = DataSource.builder()
                .enterpriseCode("liu_" + enterprise.getId())
                .ip("192.168.116.129")
                .port(3306 +enterprise.getId()).build();
        dataSourceService.save(build);
        rabbitTemplate.convertAndSend("liu-vehicle-exchange",build,message ->{
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            //设置消息延迟时间为5秒
            message.getMessageProperties().setDelay(5000);
            return message;
        } );
        // 7.关闭HttpURLConnection连接
        httpConn.disconnect();
    }

}
